#
# cached_multi_loc_prereq.mk
#
# Prerequisites used from local cache, and can be copied from
# multiple source directories, in a preferred order.
#
# Copyright (c) 2015, 2016 First Silicon Designs, LLC.
#
# The "Software" in the following 2 paragraphs refers to this file containing
# the code to "cached_multi_loc_prereq.mk" example Makefile.
#
# The "Software" does NOT refer to any other files which you
# may have received alongside this file (e.g. as part of a larger project that
# incorporates gen_file_if_different).
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software, to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the following
# conditions:
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

BROKEN        := 0

DEFAULT_PATH  := ./srcloc_default
OVERRIDE_PATH := ./srcloc_override
CACHEDIR      := ./build/cache

srcfiles_override = $(notdir $(wildcard $(OVERRIDE_PATH)/*.tcl))
srcfiles_default  = $(filter-out $(srcfiles_override),\
                      $(notdir $(wildcard $(DEFAULT_PATH)/*.tcl)))

src_prereqs  = $(addprefix $(DEFAULT_PATH)/,$(srcfiles_default))
src_prereqs += $(addprefix $(OVERRIDE_PATH)/,$(srcfiles_override))

all_direct: $(src_prereqs)
	@echo "Executing recipe for $@..."
	@echo "No extra steps for using prerequisites in-place"

ifeq ($(BROKEN),1)
srcs_default  = $(addprefix $(DEFAULT_PATH)/,$(srcfiles_default))
srcs_override = $(addprefix $(OVERRIDE_PATH)/,$(srcfiles_override))
cache_files   = $(addprefix $(CACHEDIR)/,$(srcfiles_default) $(srcfiles_override))
# This will not work.  "foreach" cannot tolerate unescaped newlines
# in the argument list, but escaping the newlines causes the lines
# to be concatenated, causing the recipe commands and command args
# to become unintended prerequisites of the intended target.
$(foreach p,$(srcs_override) $(srcs_default),$(eval\
$(CACHEDIR)/$(notdir $(p)): $(p)\
	@echo "Executing recipe for $$@..."\
	cp $$< $$@))
else
# parameter 1 is a local cache location $(CACHEDIR)
# parameter 2 is a bare file name
# parameter 3 is the source directory for the file named in $(2)
define single_prereq_rule
cache_files += $(1)/$(2)
$(1)/$(2) : $(3)/$(2) | $(1)
	cp $$< $$@
endef

$(foreach p,$(srcfiles_override),\
          $(eval $(call single_prereq_rule,\
                        $(CACHEDIR),$(p),$(OVERRIDE_PATH))))
$(foreach p,$(srcfiles_default),\
          $(eval $(call single_prereq_rule,\
                        $(CACHEDIR),$(p),$(DEFAULT_PATH))))
$(CACHEDIR):
	mkdir -p $@
endif

all_cache: $(cache_files)
	@echo "Executing recipe for $@..."

printvar-%:
	@echo "$*" = "$($(*))"

clean:
	rm $(CACHEDIR)/*
